/* 
 * CSE142, Spring 1999
 * Homework 4
 * Traffic animation
 * Richard Morris
 * Section AG
 * rakira@u.washington.edu
 */


#include "gp142.h"
#include <stdio.h>
#include <math.h>
#include <time.h>

/* Logical constants */
#define TRUE    		1
#define FALSE   		0

/* 
 * For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.  The
 * constant FILL is used in function calls when a solid
 * shape is desired.
 */
#define FILL 0

/* Menu buttons: */

/* Top-left coordinates of button palette */
#define MENU_TOP		(GP142_YMAX - 5)
#define MENU_LEFT		(-GP142_XMAX + 5)

/* Height and width of buttons in pixels */
#define BUTTON_HEIGHT  	25
#define BUTTON_WIDTH 	82

/* Number of buttons */
#define NUM_BUTTONS 	11

/*max number of cars*/
#define MAXCARS			100
#define ZERO			0


/* Possible locations of mouse clicks: */
/*   Mouse click in some button */
#define BUTTON_CLICK 1

/*   Mouse click elsewhere */
#define OTHER_CLICK 0

/* Coordinates of objects in the drawing */
/* y-coordinate of the top of the ground in the landscape */
#define HORIZON (- 2 * (GP142_YMAX / 10))

/* Vertical center and width of highway */
#define HIGHWAY_CENTER ((-GP142_YMAX + HORIZON) / 3)
#define HIGHWAY_WIDTH  10

/* Height and width of a single car */
#define CAR_HEIGHT 24
#define CAR_WIDTH  40

/*Height and width and speed of a single x-wing*/
#define	X_HEIGHT 20
#define X_WIDTH  60
#define LIGHT_SPEED		120
#define IMPULSE_SPEED	20


/********************************************************************

     FUNCTION PROTOTYPES

********************************************************************/

/* Initialize random number generator.  [Ignore or		*/
/* remove if your program doesn't use random numbers.]	*/
void initialize_random_numbers();

/* Draw the background, including the sun at coordinates sun_x and
 sun_y */
void draw_landscape(int sun_x, int sun_y);

/* Draw a retangular city*/
void draw_rectangle(int x1, int x2, int y2);
void rectangle_outline(int x1, int x2, int y2);

/* Draw the buttons */
void draw_buttons();

/* Return kind of mouse click (in a button, etc.) */
int  classify_mouse_click(int mouse_x, int mouse_y);

/* Process button click at location x, y.  Update car speed,  */
/* direction, color, and whether traffic is moving as needed. */
void handle_button(int x, int y, int *traffic_moving, int *car_speed,
				   int *car_direction, int *car_color, 
				   int *x_wing_direction_y1, int *x_wing_speed,
				   int *numcars);

/* Update sun coordinates sun_x, sun_y for one tick */
void move_sun(int *sun_x, int *sun_y);

/* Update all vehicle x, y coordinates, given current x, y */
/* location, direction (+1 or -1), and speed in pixels per tick. */
void move_traffic(int x[], int *y, int *direction, int speed,
				  int numcars);

/*X-WING PATH */	
void move_x_wing(int *x1, int *y1, int *direction_x1, 
				 int speed);

/* draws a lamp post */
void draw_lamp_post(int x1, int fact);

/* draw all planes and jets */
void draw_planes(int x1, int y1, int color,
				 int car_direction_traffic);

/* Draw all vehicles given their current x, y coordinates and color*/
void draw_traffic(int x[], int y, int color,
				  int car_direction_traffic,
				  int numcars);

/* Draw a single x-wing of the given color+1 with horizontal*/
/* center x and bottom y*/
void draw_x_wing(int x, int y, int color, int car_direction_x_wing);

/* Draw a single car of the given color with horizontal */
/* center x and bottom y*/
void draw_car(int x, int y, int color, int car_direction_car,
			  int fac);

/* Display error message if internal error is detected. */
void something_is_wrong (void);


/*******************************************************************

     MAIN

********************************************************************/

/*
 * Main program contents:
 *   - Major program variables that persist throughout execution
 *   - Event loop that processes GP142 events until the user 
 *	   quits, and updates variables and redraws the window as 
 *     needed.
 */
int main(void)
{
	/* Event handling and buttons: */
	int  quit;				/* = "user has selected quit"		*/
	int  event;				/* the most recent GP142 event		*/
	char key_pressed;		/* last keyboard character from key */
								/*event	*/
	int  mouse_x, mouse_y;	/* x and y coordinates of latest mouse */
								/*event*/
	int  click_kind;		/* kind of mouse click (button, etc.)*/

	/* Information about the sun: */
	int sun_x, sun_y;			/* screen x and y coordinates */

	/* Information about vehicles in the animation:				*/
	/* [Only one car now - will need to be changed to add more]	*/
	int car_x[MAXCARS]; /* current x position of the cars */
	int car_y;		/* current y position of the cars		*/
	int car_direction;		/* current direction: +1=right, -1=left*/
	int car_color;			/* color of the car					*/
	int car_speed;			/* speed in pixels per tick whentraffic*/
								/*moving	*/	
	int numcars;			/* numbers of cars */
	int traffic_moving;		/* = "traffic is currently moving"	*/
	int x_wing_x1, x_wing_y1; /* x_wing x and y coordinates */
	int x_wing_direction_x1; /* x_wing velocity direction */
	int x_wing_speed;	/* x wing speed */
	int i;				/* counter variable */
	initialize_random_numbers();
	
	/* Initialize graphics package, clear the window,	*/
	/* turn logging off, and start animation.			*/
	GP142_open();
	GP142_logging(LOG_OFF);
	GP142_clear();
	GP142_animate(ANI_RUN);

	/* set initial location of the sun */
	sun_x = GP142_XMAX - 50;
	sun_y = GP142_YMAX - 35;

	/* set initial conditions for the car */
	for (i =0; i <MAXCARS; i++){
		car_x[i] = -GP142_XMAX;
	}

	car_y = HIGHWAY_CENTER;
	car_direction = -1;
	car_speed = 6;
	car_color = BLACK;
	traffic_moving = TRUE;
	numcars = 0;
	
	// set up for x_wing
	x_wing_x1 = -GP142_XMAX;
	x_wing_y1 = GP142_YMAX * 0;
	x_wing_speed = 20;
	x_wing_direction_x1 = +1;

	/* Draw initial landscape, buttons, and traffic */
	draw_landscape(sun_x, sun_y);
	draw_buttons( );
	draw_traffic(car_x, car_y, car_color, car_direction, numcars);
	draw_planes(x_wing_x1, x_wing_y1, car_color, 
				x_wing_direction_x1);

	/*
	 * Main event loop:
	 * ---- ----- -----
	 * Wait for the next user action, decode it, and call an
	 * appropriate function to handle it.  Repeat until the
	 * user selects quit.
	 */
	quit = FALSE;
	while (!quit) {
		
		/* get next event */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_pressed);
		
		/* perform appropriate action depending on kind of event */
		switch (event) {
		case GP142_QUIT:
			/* Quit selected */
			quit = TRUE;
			break;
			
		case GP142_KBD:
			/* Key pressed.  Not used now.  If the program should*/
			/* react to keyboard events, insert something*/
				/*appropriate	*/
			/* here.  Note that scanf CANNOT be used with GP142.*/
			break;
			
		case GP142_MOUSE:
			/* Mouse click.  Determine if in a menu button	*/
			/* or elsewhere and process accordingly.		*/
			click_kind = classify_mouse_click(mouse_x, mouse_y);
			if (click_kind == BUTTON_CLICK) {
				handle_button(mouse_x, mouse_y, &traffic_moving,
								&car_speed, &car_direction, 
								&car_color, &x_wing_direction_x1,
								&x_wing_speed, &numcars);
				} else {
				/* insert code here (and additional else-if 
				statements) to react to mouse clicks elsewhere 
				on the screen */
			}
			break;
			
		case GP142_PERIODIC:
			/* Timer interval event.  Update everything	*/
			/* that needs to change as time advances.	*/
			move_sun(&sun_x, &sun_y);
			if (traffic_moving){
				move_traffic(car_x, &car_y, &car_direction, 
							car_speed, numcars);
				
				move_x_wing(&x_wing_x1, &x_wing_y1, 
							&x_wing_direction_x1, 
							 x_wing_speed);
			}
			break;
			
		default:
			/* unknown event */
			break;
			
		}  /*end switch */
		
		/* Redraw everything */
		draw_landscape(sun_x, sun_y);
		draw_buttons();
		draw_traffic(car_x, car_y, car_color, car_direction,
					 numcars);
		draw_planes(x_wing_x1, x_wing_y1, car_color, 
				x_wing_direction_x1);
		
	}  /* end event loop */
	
	
	/* Termination: shut down graphics window and exit */
	GP142_close();
	return 0;
}  /*end main */


/********************************************************************

     BACKGROUND LANDSCAPE

********************************************************************/


/* Draw the background, including the sun at coordinates sun_x and 
sun_y */
void draw_landscape(int sun_x, int sun_y)
{
	int text_offset = -GP142_XMAX + 15;	/* horizontal offset of */
											/*text */
	int text_size   = 12;				/* size of text font */



	if(sun_x < -GP142_XMAX -30){

		/*for night time*/
		/* Draw the sky */
		GP142_rectangleXY(BLACK, -GP142_XMAX, -GP142_YMAX,
			GP142_XMAX, GP142_YMAX, FILL);
	
		/* Draw the moon */
		GP142_circleXY(WHITE, sun_x, sun_y , 20);
	
		/* Draw the ground, highway, and text legend */
		GP142_rectangleXY(BROWN, -GP142_XMAX, -GP142_YMAX,
			GP142_XMAX, HORIZON, FILL);
		GP142_rectangleXY(23, 
						-GP142_XMAX, HIGHWAY_CENTER - 
										HIGHWAY_WIDTH,
						 GP142_XMAX, HIGHWAY_CENTER +
										HIGHWAY_WIDTH,
						 FILL);
		/* draws lamp posts */
		
		draw_lamp_post( 3, 1);
		draw_lamp_post( 2, 1);
		draw_lamp_post( 1, 1);
		draw_lamp_post( 4, 1);
		draw_lamp_post( 0, 1);
		draw_lamp_post( -1, 1);
		draw_lamp_post( -2, 1);
		draw_lamp_post( -3, 1);
		draw_lamp_post( -4, 1);
						
		/* draws the light of lamp posts */
		GP142_circleXY(YELLOW, 
						-GP142_XMAX * 3/4, HIGHWAY_CENTER,
						HIGHWAY_WIDTH);
		GP142_circleXY(YELLOW, 
						-GP142_XMAX * 2/4, HIGHWAY_CENTER,
						HIGHWAY_WIDTH);
		GP142_circleXY(YELLOW, 
						-GP142_XMAX * 1/4, HIGHWAY_CENTER,
						HIGHWAY_WIDTH);
		GP142_circleXY(YELLOW, 
						-GP142_XMAX * 1, HIGHWAY_CENTER,
						HIGHWAY_WIDTH);
		GP142_circleXY(YELLOW, 
						-GP142_XMAX * 0, HIGHWAY_CENTER,
						HIGHWAY_WIDTH);
		GP142_circleXY(YELLOW, 
						GP142_XMAX * 1/4, HIGHWAY_CENTER,
						HIGHWAY_WIDTH);
		GP142_circleXY(YELLOW, 
						GP142_XMAX * 2/4, HIGHWAY_CENTER,
						HIGHWAY_WIDTH);
		GP142_circleXY(YELLOW, 
						GP142_XMAX * 3/4, HIGHWAY_CENTER,
						HIGHWAY_WIDTH);
		GP142_circleXY(YELLOW, 
						GP142_XMAX * 1, HIGHWAY_CENTER,
						HIGHWAY_WIDTH);
						
		/* draw center line of highway */
		GP142_lineXY (BLACK,
						-GP142_XMAX, HIGHWAY_CENTER,
						 GP142_XMAX, HIGHWAY_CENTER,
						 1);

		/*EXPRESS LANE*/

		/* draws lamp posts  twice as large*/
		draw_lamp_post( 4, 2);
		draw_lamp_post( 2, 2);
		draw_lamp_post( 0, 2);
		draw_lamp_post( -2, 2);
		draw_lamp_post( -4, 2);

		GP142_rectangleXY(23, 
						-GP142_XMAX, HIGHWAY_CENTER*2 - 
										HIGHWAY_WIDTH*2,
						 GP142_XMAX, HIGHWAY_CENTER*2 + 
										HIGHWAY_WIDTH*2,
						 FILL);

		/* draws light spots */
		GP142_circleXY(YELLOW, 
					-	GP142_XMAX * 1, 2* HIGHWAY_CENTER,
						2* HIGHWAY_WIDTH);
		GP142_circleXY(YELLOW, 
						-GP142_XMAX * 2/4, 2* HIGHWAY_CENTER,
						2* HIGHWAY_WIDTH);
		GP142_circleXY(YELLOW, 
						-GP142_XMAX * 0, 2* HIGHWAY_CENTER,
						2* HIGHWAY_WIDTH);
		GP142_circleXY(YELLOW, 
						GP142_XMAX * 2/4, 2* HIGHWAY_CENTER,
						2* HIGHWAY_WIDTH);
		GP142_circleXY(YELLOW, 
						GP142_XMAX * 1, 2* HIGHWAY_CENTER,
						2* HIGHWAY_WIDTH);
	
		/* draw center lin for  high way */		
		GP142_lineXY (BLACK,
						-GP142_XMAX, HIGHWAY_CENTER*2,
						 GP142_XMAX, HIGHWAY_CENTER*2,
						 1);

		GP142_printfXY (YELLOW, text_offset,-GP142_YMAX+10,text_size,
			"Click a button to control the car.");

		
	/*Draw some mountains */
	GP142_triangleXY(BLACK,
		-GP142_XMAX, HORIZON,
		GP142_XMAX, HORIZON,
		0, (4 * GP142_YMAX)/5 ,
		FILL);
	
	GP142_triangleXY(OLIVE,
		-( 5 * GP142_XMAX/4), HORIZON,
		(GP142_XMAX)/3  , HORIZON,
		-(3* GP142_XMAX)/4, (2 * GP142_YMAX)/3 ,
		FILL);
	
	GP142_triangleXY(23,
		-GP142_XMAX / 3 , HORIZON,
		GP142_XMAX, HORIZON,
		GP142_XMAX, (2 * GP142_YMAX)/3 ,
		FILL);

	}

	
	/* for the day time */

	else{

		/* Draw the sky */
		GP142_rectangleXY(ICE_BLUE, -GP142_XMAX, -GP142_YMAX,
			GP142_XMAX, GP142_YMAX, FILL);
	
		/* Draw the sun */
		GP142_circleXY(YELLOW, sun_x, sun_y , 20);
	
		/* Draw the ground, highway, and text legend */
		GP142_rectangleXY(BROWN, -GP142_XMAX, -GP142_YMAX,
			GP142_XMAX, HORIZON, FILL);

		/* draws lamp posts */
		draw_lamp_post( 3, 1);
		draw_lamp_post( 2, 1);
		draw_lamp_post( 1, 1);
		draw_lamp_post( 4, 1);
		draw_lamp_post( 0, 1);
		draw_lamp_post( -1, 1);
		draw_lamp_post( -2, 1);
		draw_lamp_post( -3, 1);
		draw_lamp_post( -4, 1);

		GP142_rectangleXY(WHITE, 
						-GP142_XMAX, HIGHWAY_CENTER - HIGHWAY_WIDTH,
						 GP142_XMAX, HIGHWAY_CENTER + HIGHWAY_WIDTH,
						 FILL);
		/* draw center line for highway */
		GP142_lineXY (BLACK,
						-GP142_XMAX, HIGHWAY_CENTER,
						 GP142_XMAX, HIGHWAY_CENTER,
						 1);

		/* espress lane */

		/* draws lamp posts  twice as large*/

		draw_lamp_post( 4, 2);
		draw_lamp_post( 2, 2);
		draw_lamp_post( 0, 2);
		draw_lamp_post( -2, 2);
		draw_lamp_post( -4, 2);
		GP142_rectangleXY(WHITE, 
						-GP142_XMAX, HIGHWAY_CENTER*2 - 
										HIGHWAY_WIDTH*2,
						 GP142_XMAX, HIGHWAY_CENTER*2 + 
										HIGHWAY_WIDTH*2,
						 FILL);

		/* draw center line for highway */		
		GP142_lineXY (BLACK,
						-GP142_XMAX, HIGHWAY_CENTER*2,
						 GP142_XMAX, HIGHWAY_CENTER*2,
						 1);

		GP142_printfXY (BLACK, text_offset, -GP142_YMAX+10,text_size,
			"Click a button to control the car.");

		
	/*Draw some mountains */
	GP142_triangleXY(22,
		-GP142_XMAX, HORIZON,
		GP142_XMAX, HORIZON,
		0, (4 * GP142_YMAX)/5 ,
		FILL);
	
	GP142_triangleXY(20,
		-( 5 * GP142_XMAX/4), HORIZON,
		(GP142_XMAX)/3  , HORIZON,
		-(3* GP142_XMAX)/4, (2 * GP142_YMAX)/3 ,
		FILL);
	
	GP142_triangleXY(3,
		-GP142_XMAX / 3 , HORIZON,
		GP142_XMAX, HORIZON,
		GP142_XMAX, (2 * GP142_YMAX)/3 ,
		FILL);
	}


	

} 
/*end draw_landscape */




  

/********************************************************************

     BUTTONS AND MOUSE CLICKS

********************************************************************/


/* Draws the buttons */
void draw_buttons( )
{
	int i;							/* counter variable */
	int text_offset = MENU_LEFT+10;	/* offset of button text in */
										/*button */
	int font_size = 12;				/* size of button text */
	
	/* draw rectangles for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		GP142_rectangleXY(BLACK,
			MENU_LEFT,
			MENU_TOP - i * BUTTON_HEIGHT,
			MENU_LEFT + BUTTON_WIDTH,
			MENU_TOP - (i + 1)*BUTTON_HEIGHT, 1);
	}
	
	/* display button text */
	i = MENU_TOP - (BUTTON_HEIGHT/2) - (font_size/2);
	GP142_printfXY(4, text_offset, i, font_size, "Go");	
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(4, text_offset, i, font_size, "Stop");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(4, text_offset, i, font_size, "Faster");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(4, text_offset, i, font_size, "Slower");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(4, text_offset, i, font_size, "Reverse");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(4, text_offset, i, font_size, "New Color");

	i -= BUTTON_HEIGHT;
	GP142_printfXY(4, text_offset, i, font_size, "X Reverse");

	i -= BUTTON_HEIGHT;
	GP142_printfXY(4, text_offset, i, font_size, "X Light Speed");

	i -= BUTTON_HEIGHT;
	GP142_printfXY(4, text_offset, i, font_size, "Impulse Speed");

	i -= BUTTON_HEIGHT;
	GP142_printfXY(4, text_offset, i, font_size, "Add Car");

	i -= BUTTON_HEIGHT;
	GP142_printfXY(4, text_offset, i, font_size, "Clear Cars");
}

/* Classify mouse click at (mouse_x,mouse_y) and return kind */
int  classify_mouse_click(int mouse_x, int mouse_y)
{
	if ((mouse_x > MENU_LEFT) &&
		(mouse_x < MENU_LEFT + BUTTON_WIDTH) &&
		(mouse_y < MENU_TOP) &&
		(mouse_y > (MENU_TOP - NUM_BUTTONS*BUTTON_HEIGHT)))
		return BUTTON_CLICK;
	else
		return OTHER_CLICK;
}



/* Process button click at location x, y.  Update car speed,	*/
/* direction, color, and whether traffic is moving as needed.	*/
void handle_button(int x, int y, int * traffic_moving, 
				   int * car_speed, int * car_direction,
				   int * car_color, int * x_wing_direction_x1,
				   int * x_wing_speed, int * numcars)
{
	int buttonNum;		/* number of selected button */




	/* Complain if x coordinate couldn't be inside a button */
	if (x < MENU_LEFT || x > MENU_LEFT + BUTTON_WIDTH) {
		something_is_wrong();
		return;
	}
	
	/* Convert y coordinate to button number from top. */
	buttonNum = (MENU_TOP - y)/BUTTON_HEIGHT;	 

	
	switch(buttonNum) {
	case 0:
		/* go */
		*traffic_moving = TRUE;
		/* something missing here? */
		break;
	case 1:
		/* stop */
		*traffic_moving = FALSE;
		break;
		
	case 2:
		/* speed up */
		*car_speed = *car_speed + 1;
		
		*x_wing_speed = *x_wing_speed + 2;

		break;
	case 3:
		/* slow down */
		*car_speed = *car_speed - 1;
		if (*car_speed < 0)
			*car_speed = 0;

		*x_wing_speed = *x_wing_speed - 2;
		if (*x_wing_speed < 0)
			*x_wing_speed = 0;
		break;
		
	case 4:
		/* reverse */
		*car_direction = - *car_direction;
		break;
	case 5:
		/* change car color */
		*car_color = (*car_color+1) % MAX_COLORS;
		break;
	case 6:
		/*  X-Wing reverse */
		*x_wing_direction_x1 = - *x_wing_direction_x1;
		break;
	case 7:
		/*Light speed*/
		*x_wing_speed = LIGHT_SPEED;
		break;
	case 8:
		/*Brings x_wing back to normal speed*/
		*x_wing_speed = IMPULSE_SPEED;
		break;
	case 9:
		/*adds a car*/
		*numcars = *numcars + 1;
		break;
	case 10:
		/*clears cars */
		*numcars = ZERO;
	default:
		/*Bad news if this is ever reached! */
		something_is_wrong();
		break;
	}
}



/********************************************************************

     TRAFFIC AND OTHER ANIMATED OBJECTS

********************************************************************/


/* Update sun coordinates sun_x, sun_y by one tick */
void move_sun(int *sun_x, int *sun_y) {
	/* Move the sun to the left.  If it reaches the edge, move it */
	/* to the other side and down a bit. */
	(*sun_x) = *sun_x - 5;
	if (*sun_x < -GP142_XMAX -30 -  2 * GP142_XMAX) {
		*sun_x = - *sun_x  -30 -  2 * GP142_XMAX;
	}
}

/* X-WING PATH	*/
void move_x_wing(int *x1, int *y1, int *direction_x1,
				 int speed){
	
	int temp_x1;  /* temp to hold the next x value */

	temp_x1 = *x1 + (*direction_x1 * speed);

	/* checks to see if the x_ wing needs to return to the screen */
	if (abs(temp_x1) > GP142_XMAX + 10 * X_WIDTH)
		*x1 = - (*x1);
	else
		*x1 = temp_x1;

}
	
/* Update all vehicle x, y coordinates, given current x, y */
/* location, direction (+1 or -1), and speed in pixels per tick, */
void move_traffic(int car_x[], int *y, int *direction, int speed,
				  int numcars) {
	/* Note: If there is more than one vehicle, this should */
	/* be updated to adjust all vehicle coordinates */

	int trial_top[MAXCARS];	/* new value for x coord. in top lane*/
	int trial_bot[MAXCARS]; /* new value for x coord. in bot lane*/
	int i;			/* case variable */
	
	/* move vehicle.  If it moves off the edge, reverse edge */
	for (i = 0; i <= numcars; i++){

		/* moves cars on the top side */
		if (i%2 == 0){
			trial_top[i] = car_x[i] + (*direction * speed);

			if (abs(trial_top[i]) > GP142_XMAX + CAR_WIDTH/2)
				car_x[i] = - (car_x[i]);
		
				else
					car_x[i] = trial_top[i];
		}

		else{ /* moves cars on the bottom side */

				trial_bot[i] = car_x[i] + (-*direction * speed);

				if (abs(trial_bot[i]) > GP142_XMAX + CAR_WIDTH/2)
					car_x[i] = - (car_x[i]);
		
					else
						car_x[i] = trial_bot[i];
		}
	}

}

/* draws a lamp post */
void draw_lamp_post(int x1, int fact){
	GP142_lineXY(BLACK,
				GP142_XMAX * x1/4 , HIGHWAY_CENTER * fact + 
										HIGHWAY_WIDTH * fact,
				GP142_XMAX * x1/4 , HIGHWAY_CENTER * fact +
										HIGHWAY_WIDTH * 4 * fact,
				1 * fact);
	GP142_circleXY( BLACK,
				GP142_XMAX * x1/4 , HIGHWAY_CENTER * fact + 
										HIGHWAY_WIDTH * 4 * fact,
				2 * fact);
}


	
/* draw x-wing in non-linear motion) */
/* drawn in formation for added affect */
void draw_planes(int x1, int y1, int color, int x_wing_direction_x){
	if (x_wing_direction_x == +1){
		draw_x_wing(x1 - (X_WIDTH * 10/9) , GP142_YMAX* 1/2 + 
			(X_HEIGHT ), 9, x_wing_direction_x);
		draw_x_wing(x1 , GP142_YMAX* 1/2, 23,
					x_wing_direction_x);
		draw_x_wing(x1 - (X_WIDTH * 10/9) , GP142_YMAX* 1/2 - 
			(X_HEIGHT ), 9, x_wing_direction_x);
	}
	else{
		draw_x_wing(x1 + (X_WIDTH * 10/9) , GP142_YMAX* 1/2 - 
			(X_HEIGHT ), 9, x_wing_direction_x);
		draw_x_wing(x1 , GP142_YMAX* 1/2 , 23,
					x_wing_direction_x);
		draw_x_wing(x1 + (X_WIDTH * 10/9) , GP142_YMAX* 1/2 + 
			(X_HEIGHT ), 9, x_wing_direction_x);
	}
}

/* Draw all cars given their current x, y coordinates and color*/
void draw_traffic(int x[], int y, int color, 
				  int car_direction_traffic,
				  int numcars) {
	
	int i = 0;	/* counter variable */
	int car_color;	/* cars color */
	int car_y_position;	/* cars y coordinate */
	int car_dir;	/* direction of the car */
	int express_y;	/* the y coordinate of the express lanes */
	int fac;		/* the multiplication factor to make the cars */

	fac = 1;	

	/* draws a desired amount of cars*/
		for (i = 0; i < numcars; i++) {
		
			car_color = color + i;
	
			if ( car_color > 24){
				car_color = color - 1;
			}

			if (car_color < 0){
				car_color = color + 1;
			}

		/* checks to see what lane they are in*/
			if (i%2 == 0){
				car_y_position = y;
				car_dir = car_direction_traffic;
				
				draw_car(x[i] , car_y_position,
				car_color , car_dir, 1 );
			}

		}
	

		for (i = 0; i < numcars; i++) {
		
			car_color = color + i;

			if ( car_color > 20){
				car_color = color - 1;
			}

			if (car_color < 0){
				car_color = color + 1;
			}

			/* checks to see what lane they are in*/
			if (i%2 != 0){
				car_y_position = y - HIGHWAY_WIDTH;
				car_dir = - car_direction_traffic;

				draw_car(x[i] , car_y_position,
				car_color , car_dir ,1);
			}

		}
		

		/*EXPRESS LANE*/

		for (i = 0; i < numcars; i++) {
			if ( i % 3 == 0){
			
			express_y = HIGHWAY_CENTER * 2;
		
			car_color = color + i;

			if ( car_color > 20){
				car_color = color - 1;
			}

			if (car_color < 0){
				car_color = color + 1;
			}

			/* checks to see what lane they are in*/

			if (i%2 == 0){
				express_y = express_y;
				car_dir = car_direction_traffic;

				draw_car(x[i] , express_y,
				car_color , car_dir, 2);

			}
			}
		}

		/*ESPRESS LANE TWO*/
		for (i = 0; i < numcars; i++) {
			if ( i % 3 == 0){
			
			express_y = HIGHWAY_CENTER * 2;
		
			car_color = color + i;

			if ( car_color > 20){
				car_color = color - 1;
			}

			if (car_color < 0){
				car_color = color + 1;
			}

			/* checks to see what lane they are in*/

			if (i%2 != 0){
				express_y = express_y - HIGHWAY_WIDTH * 2;
				car_dir = - car_direction_traffic;

				draw_car(x[i] , express_y,
				car_color , car_dir, 2);

			}
			}
		}
	
}

/* Draw a single x-wing with horizontal*/
/* center x and bottom y*/

void draw_x_wing(int x, int y, int color, int car_direction_x_wing){
	if (car_direction_x_wing == 1){
			
		//MAIN BODY
		GP142_rectangleXY(color, x, y + X_HEIGHT * 1/8,
						x - X_WIDTH * 1/3, y + X_HEIGHT * 1/2, FILL);
		
		GP142_rectangleXY(color, x - X_HEIGHT * 1/8, y + X_HEIGHT * 
						1/8, x - X_WIDTH * 1/3, y, FILL);
		
		GP142_triangleXY(color, x, y + X_HEIGHT * 1/8,
						x - X_HEIGHT * 1/8, y + X_HEIGHT * 1/8,
						x - X_HEIGHT * 1/8, y, FILL);
		
		GP142_triangleXY(color, x + X_WIDTH * 5/9, y + X_HEIGHT *1/8,
						x + X_WIDTH * 5/9, y + X_HEIGHT * 3/8,
						x + X_WIDTH * 7/9, y + X_HEIGHT * 3/16,FILL);

		GP142_rectangleXY(color, x, y + X_HEIGHT * 3/8,
						x + X_WIDTH * 5/9, y + X_HEIGHT *1/8, FILL);
		
		//COCKPIT
		GP142_rectangleXY(ICE_BLUE, x, y + X_HEIGHT*1/2,
						x + X_WIDTH * 1/9, y + X_HEIGHT * 3/8, FILL);
		
		GP142_triangleXY(ICE_BLUE, x + X_WIDTH * 1/9, y+X_HEIGHT*1/2,
						x + X_WIDTH * 1/9, y + X_HEIGHT * 3/8,
						x + X_WIDTH * 2/9, y + X_HEIGHT * 3/8, FILL);

		GP142_rectangleXY(BLACK, x, y + X_HEIGHT*1/2,
						x + X_WIDTH * 1/9, y + X_HEIGHT * 3/8, 1);
		
		GP142_triangleXY(BLACK, x + X_WIDTH * 1/9, y + X_HEIGHT*1/2,
						x + X_WIDTH * 1/9, y + X_HEIGHT * 3/8,
						x + X_WIDTH * 2/9, y + X_HEIGHT * 3/8, 1);
		//WINGS

		GP142_rectangleXY(color, x - X_WIDTH * 1/9, y + X_HEIGHT*3/8,
						x - X_WIDTH * 5/18, y + X_HEIGHT * 3/4,FILL);

		GP142_rectangleXY(color, x - X_WIDTH * 1/9, y + X_HEIGHT*1/8,
						x - X_WIDTH * 5/18, y - X_HEIGHT * 1/4,FILL);

		//OUTLINE

		GP142_rectangleXY(BLACK, x - X_WIDTH * 1/9, y + X_HEIGHT*3/8,
						x - X_WIDTH * 5/18, y + X_HEIGHT * 3/4, 1);

		GP142_rectangleXY(BLACK, x - X_WIDTH * 1/9, y + X_HEIGHT*1/8,
						x - X_WIDTH * 5/18, y - X_HEIGHT * 1/4, 1);

		
		//LASERS
		
		GP142_rectangleXY(color, x - X_WIDTH * 1/12, y + X_HEIGHT * 
						13/16, x - X_WIDTH * 11/36, y + X_HEIGHT * 
						3/4, FILL);

		GP142_rectangleXY(color, x - X_WIDTH * 1/12, y - X_HEIGHT *
						1/4, x - X_WIDTH * 11/36, y - X_HEIGHT * 
						5/16 , FILL);
		
		GP142_lineXY(BLACK, x - X_WIDTH * 1/12, y + X_HEIGHT * 25/32,
						x + X_WIDTH * 1/3, y + X_HEIGHT * 25/32,1);

		GP142_lineXY(BLACK, x - X_WIDTH * 1/12, y - X_HEIGHT * 9/32,
						x + X_WIDTH * 1/3, y - X_HEIGHT * 9/32,1);
		
			//OUTLINE

		GP142_rectangleXY(BLACK, x - X_WIDTH * 1/12, y + X_HEIGHT * 
						13/16, x - X_WIDTH * 11/36, y + X_HEIGHT *
						3/4, 1);

		GP142_rectangleXY(BLACK, x - X_WIDTH * 1/12, y - X_HEIGHT * 
						1/4, x - X_WIDTH * 11/36, y - X_HEIGHT * 
						5/16 , 1);
		
		//ENGINES

		GP142_circleXY (RED, x - X_WIDTH * 4/9, y + X_HEIGHT * 1/2, 
						X_HEIGHT * 1/8);
		
		GP142_circleXY (RED, x - X_WIDTH * 4/9, y, X_HEIGHT * 1/8);
		
		GP142_rectangleXY(color, x - X_WIDTH * 5/18, y + X_HEIGHT *
						3/8, x - X_WIDTH * 4/9, y + X_HEIGHT * 
						5/8 , FILL);
		
		GP142_rectangleXY(color, x - X_WIDTH * 5/18, y + X_HEIGHT *
						1/8, x - X_WIDTH * 4/9, y - X_HEIGHT * 
						1/8 , FILL);
		
			//OUTLINE

		GP142_rectangleXY(BLACK, x - X_WIDTH * 5/18, y + X_HEIGHT *
						3/8, x - X_WIDTH * 4/9, y + X_HEIGHT * 
						5/8 , 1);
		
		GP142_rectangleXY(BLACK, x - X_WIDTH * 5/18, y + X_HEIGHT * 
						1/8, x - X_WIDTH * 4/9, y - X_HEIGHT * 
						1/8 , 1);

		//REST OF OUTLINES

		GP142_lineXY(BLACK, x - X_WIDTH * 1/3, y + X_HEIGHT * 1/8,
						x - X_WIDTH * 1/3, y + X_HEIGHT * 3/8 ,1);

		GP142_lineXY(BLACK, x, y + X_HEIGHT * 1/8,
						x - X_HEIGHT * 1/8, y ,1);

		GP142_lineXY(BLACK, x - X_HEIGHT * 1/8, y,
						x - X_WIDTH * 1/9, y ,1);
	
		GP142_lineXY(BLACK, x, y + X_HEIGHT * 1/8,
						x + X_WIDTH * 5/9, y + X_HEIGHT * 1/8 ,1);

		GP142_lineXY(BLACK, x, y + X_HEIGHT * 3/8,
						x + X_WIDTH * 5/9, y + X_HEIGHT * 3/8 ,1);

		GP142_lineXY(BLACK, x + X_WIDTH * 7/9, y + X_HEIGHT * 3/16,
						x + X_WIDTH * 5/9, y + X_HEIGHT * 3/8 ,1);

		GP142_lineXY(BLACK, x + X_WIDTH * 7/9, y + X_HEIGHT * 3/16,
						x + X_WIDTH * 5/9, y + X_HEIGHT * 1/8 ,1);

		GP142_lineXY(BLACK, x , y + X_HEIGHT * 1/2,
						x - X_WIDTH * 1/9, y + X_HEIGHT * 1/2 ,1);



	}
	else{		
		/* this is for the reverse direction */

		/* MAIN BODY */
		GP142_rectangleXY(color, x, y + X_HEIGHT * 1/8,
						x + X_WIDTH * 1/3, y + X_HEIGHT * 1/2, FILL);
		
		GP142_rectangleXY(color, x + X_HEIGHT * 1/8, y + X_HEIGHT * 
					1/8, x + X_WIDTH * 1/3, y, FILL);
		
		GP142_triangleXY(color, x, y + X_HEIGHT * 1/8,
						x + X_HEIGHT * 1/8, y + X_HEIGHT * 1/8,
						x + X_HEIGHT * 1/8, y, FILL);
		
		GP142_triangleXY(color, x - X_WIDTH * 5/9, y + X_HEIGHT *1/8,
						x - X_WIDTH * 5/9, y + X_HEIGHT * 3/8,
						x - X_WIDTH * 7/9, y + X_HEIGHT * 3/16,FILL);

		GP142_rectangleXY(color, x, y + X_HEIGHT * 3/8,
						x - X_WIDTH * 5/9, y + X_HEIGHT *1/8, FILL);
		
		//COCKPIT
		GP142_rectangleXY(ICE_BLUE, x, y + X_HEIGHT*1/2,
						x - X_WIDTH * 1/9, y + X_HEIGHT * 3/8, FILL);
		
		GP142_triangleXY(ICE_BLUE, x - X_WIDTH * 1/9, y + X_HEIGHT * 
						1/2, x - X_WIDTH * 1/9, y + X_HEIGHT * 3/8,
						x - X_WIDTH * 2/9, y + X_HEIGHT * 3/8, FILL);
		
		GP142_rectangleXY(BLACK, x, y + X_HEIGHT*1/2,
						x - X_WIDTH * 1/9, y + X_HEIGHT * 3/8, 1);
		
		GP142_triangleXY(BLACK, x - X_WIDTH * 1/9, y + X_HEIGHT * 
						1/2, x - X_WIDTH * 1/9, y + X_HEIGHT * 3/8,
						x - X_WIDTH * 2/9, y + X_HEIGHT * 3/8, 1);
		//WINGS

		GP142_rectangleXY(color, x + X_WIDTH * 1/9, y + X_HEIGHT * 
						3/8, x + X_WIDTH * 5/18, y + X_HEIGHT * 3/4,
						FILL);

		GP142_rectangleXY(color, x + X_WIDTH * 1/9, y + X_HEIGHT * 
						1/8, x + X_WIDTH * 5/18, y - X_HEIGHT * 1/4, 
						FILL);

		//OUTLINE

		GP142_rectangleXY(BLACK, x + X_WIDTH * 1/9, y + X_HEIGHT * 
						3/8, x + X_WIDTH * 5/18, y + X_HEIGHT * 3/4,
						1);

		GP142_rectangleXY(BLACK, x + X_WIDTH * 1/9, y + X_HEIGHT * 
						1/8, x + X_WIDTH * 5/18, y - X_HEIGHT * 1/4,
						1);

		
		//LASERS
		
		GP142_rectangleXY(color, x + X_WIDTH * 1/12, y + X_HEIGHT *
						13/16, x + X_WIDTH * 11/36, y + X_HEIGHT * 
						3/4, FILL);

		GP142_rectangleXY(color, x + X_WIDTH * 1/12, y - X_HEIGHT *
						1/4, x + X_WIDTH * 11/36, y - X_HEIGHT * 
						5/16 , FILL);
		
		GP142_lineXY(BLACK, x + X_WIDTH * 1/12, y + X_HEIGHT * 25/32,
						x - X_WIDTH * 1/3, y + X_HEIGHT * 25/32,1);

		GP142_lineXY(BLACK, x + X_WIDTH * 1/12, y - X_HEIGHT * 9/32,
						x - X_WIDTH * 1/3, y - X_HEIGHT * 9/32,1);
		
			//OUTLINE

		GP142_rectangleXY(BLACK, x + X_WIDTH * 1/12, y + X_HEIGHT * 
						13/16, x + X_WIDTH * 11/36, y + X_HEIGHT * 
						3/4, 1);

		GP142_rectangleXY(BLACK, x + X_WIDTH * 1/12, y - X_HEIGHT *
						1/4, x + X_WIDTH * 11/36, y - X_HEIGHT *
						5/16 , 1);
		
		//ENGINES

		GP142_circleXY (RED, x + X_WIDTH * 4/9, y + X_HEIGHT * 1/2, 
						X_HEIGHT * 1/8);
		
		GP142_circleXY (RED, x + X_WIDTH * 4/9, y, X_HEIGHT * 1/8);
		
		GP142_rectangleXY(color, x + X_WIDTH * 5/18, y + X_HEIGHT * 
						3/8, x + X_WIDTH * 4/9, y + X_HEIGHT * 5/8 ,
						FILL);
		
		GP142_rectangleXY(color, x + X_WIDTH * 5/18, y + X_HEIGHT *
						1/8, x + X_WIDTH * 4/9, y - X_HEIGHT * 1/8 ,
						FILL);
		
			//OUTLINE

		GP142_rectangleXY(BLACK, x + X_WIDTH * 5/18, y + X_HEIGHT *
						3/8, x + X_WIDTH * 4/9, y + X_HEIGHT * 5/8 ,
						1);
		
		GP142_rectangleXY(BLACK, x + X_WIDTH * 5/18, y + X_HEIGHT *
						1/8, x + X_WIDTH * 4/9, y - X_HEIGHT * 1/8 ,
						1);

		//REST OF OUTLINES
		GP142_lineXY(BLACK, x + X_WIDTH * 1/3, y + X_HEIGHT * 1/8,
						x + X_WIDTH * 1/3, y + X_HEIGHT * 3/8 ,1);

		GP142_lineXY(BLACK, x, y + X_HEIGHT * 1/8,
						x + X_HEIGHT * 1/8, y ,1);

		GP142_lineXY(BLACK, x + X_HEIGHT * 1/8, y,
						x + X_WIDTH * 1/9, y ,1);
	
		GP142_lineXY(BLACK, x, y + X_HEIGHT * 1/8,
						x - X_WIDTH * 5/9, y + X_HEIGHT * 1/8 ,1);

		GP142_lineXY(BLACK, x, y + X_HEIGHT * 3/8,
						x - X_WIDTH * 5/9, y + X_HEIGHT * 3/8 ,1);

		GP142_lineXY(BLACK, x - X_WIDTH * 7/9, y + X_HEIGHT * 3/16,
						x - X_WIDTH * 5/9, y + X_HEIGHT * 3/8 ,1);

		GP142_lineXY(BLACK, x - X_WIDTH * 7/9, y + X_HEIGHT * 3/16,
						x - X_WIDTH * 5/9, y + X_HEIGHT * 1/8 ,1);

		GP142_lineXY(BLACK, x , y + X_HEIGHT * 1/2,
						x + X_WIDTH * 1/9, y + X_HEIGHT * 1/2 ,1);
	}
}
/* Draw a single car of the given color with horizontal */
/* center x and bottom y*/


void draw_car(int x, int y, int color, int car_direction_car, 
			  int fac) {
	
	GP142_rectangleXY(color, x - CAR_WIDTH/2 *fac, y + CAR_HEIGHT*1/2 
					* fac, x + CAR_WIDTH/2 *fac, y + CAR_HEIGHT*3/4 *
					fac, FILL);

	if (car_direction_car == 1){
		
		GP142_rectangleXY(color, x, y + CAR_HEIGHT*3/4 * fac,
						x + CAR_WIDTH/6 * fac, y + CAR_HEIGHT * fac,
						FILL);

		GP142_triangleXY(ICE_BLUE, x + CAR_WIDTH/6 * fac,
						y + CAR_HEIGHT * fac, x + CAR_WIDTH/6 * fac,
						y + CAR_HEIGHT*3/4 * fac, x + CAR_WIDTH/4 *
						fac, y + CAR_HEIGHT*3/4 * fac, FILL);

		GP142_lineXY (BLACK, x, y + CAR_HEIGHT * fac,
						x - CAR_WIDTH/4 * fac, y + CAR_HEIGHT*3/4 *
						fac, 1 * fac);
	}
		
	else{ /* this is for the reverse direction */

		GP142_rectangleXY(color, x - CAR_WIDTH/6 * fac,
						y + CAR_HEIGHT*3/4 * fac,
						x, y + CAR_HEIGHT * fac, FILL);
	
		GP142_triangleXY(ICE_BLUE, x - CAR_WIDTH/6 * fac,
						y + CAR_HEIGHT * fac, x - CAR_WIDTH/6 * fac,
						y + CAR_HEIGHT*3/4 * fac, x - CAR_WIDTH/4 *
						fac, y + CAR_HEIGHT*3/4 * fac, FILL);

		GP142_lineXY (BLACK, x, y + CAR_HEIGHT * fac,
						x + CAR_WIDTH/4 * fac, y + CAR_HEIGHT*3/4 *
						fac, 1 * fac);
	}

	/* draws tires and wheels */
	GP142_circleXY(BLACK, x - CAR_WIDTH/4 * fac, y + CAR_HEIGHT/4 * 
				fac, CAR_HEIGHT/4 * fac);

	GP142_circleXY(BLACK, x + CAR_WIDTH/4 * fac, y + CAR_HEIGHT/4 * 
				fac, CAR_HEIGHT/4 * fac);

	GP142_circleXY(23, x - CAR_WIDTH/4 * fac, y + CAR_HEIGHT/4 * fac, 
				CAR_HEIGHT/8 * fac);

	GP142_circleXY(23, x + CAR_WIDTH/4 * fac, y + CAR_HEIGHT/4 * fac, 
				CAR_HEIGHT/8 * fac);

	GP142_circleXY(BLACK, x - CAR_WIDTH/4 * fac, y + CAR_HEIGHT/4 * 
				fac, 1 * fac);

	GP142_circleXY(BLACK, x + CAR_WIDTH/4 * fac, y + CAR_HEIGHT/4 *
				fac, 1 * fac);
}



/********************************************************************

     OTHER

********************************************************************/


/* Initialize random number generator */
/* [Ignore or delete if your program  */
/* doesn't user random numbers.] */
void initialize_random_numbers() {
	time_t t;			/* current time */
	srand (time(&t));
}



/* Display error message if internal error is detected. */
void something_is_wrong (void) {
	GP142_printfXY (RED, 0, 0, 16, "BAAAADDD NEWS!");
}
